package ru.elehant.plugin.mobilehelper

import android.accounts.AccountManager
import android.annotation.SuppressLint
import android.app.Activity
import android.content.ComponentName
import android.content.Context
import android.content.Context.POWER_SERVICE
import android.content.Intent
import android.content.SharedPreferences
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.PowerManager
import android.preference.PreferenceManager
import android.provider.Settings
import androidx.activity.result.ActivityResult
import androidx.annotation.RequiresApi
import androidx.core.app.ActivityCompat
import com.getcapacitor.JSObject
import com.getcapacitor.Plugin
import com.getcapacitor.PluginCall
import com.getcapacitor.PluginMethod
import com.getcapacitor.annotation.ActivityCallback
import com.getcapacitor.annotation.CapacitorPlugin
import com.google.android.gms.common.AccountPicker
import java.util.Locale
import com.google.android.gms.common.ConnectionResult
import com.google.android.gms.common.GoogleApiAvailability


@CapacitorPlugin(name = "MobileHelper")
class MobileHelperPlugin : Plugin() {

    private lateinit var context: Context
    private  var powerManager: PowerManager? = null
    private var googleApiAvailability: GoogleApiAvailability? = null

    override fun load() {
        context = getContext()
        powerManager = context.getSystemService(POWER_SERVICE) as PowerManager?
        googleApiAvailability = GoogleApiAvailability.getInstance();
    }

     private fun isGooglePlayServicesAvailable(): Boolean  {
         val status = googleApiAvailability?.isGooglePlayServicesAvailable(activity);
         if (status != ConnectionResult.SUCCESS) {
             return false;
         }
         return true;
     }

     @PluginMethod()
     public fun checkGooglePlay(call: PluginCall) {
         try {
             val isAvailable: Boolean = this.isGooglePlayServicesAvailable();
             val ret = JSObject()
             ret.put("isOk", isAvailable);
             call.resolve(ret);
         } catch (exception: Exception) {
             call.reject(exception.getLocalizedMessage());
         }
     }

        // Get Power Manager service

    private fun isIntentResolved(intent: Intent?): Boolean {
        return (intent != null && context.packageManager.resolveActivity(intent, PackageManager.MATCH_DEFAULT_ONLY) != null)
    }

    private fun checkMIUI(): Boolean {
        return (Build.MANUFACTURER.lowercase(Locale.getDefault()) == "xiaomi") && (
                isIntentResolved(Intent("miui.intent.action.OP_AUTO_START").addCategory(Intent.CATEGORY_DEFAULT)) ||
                isIntentResolved(Intent().setComponent(ComponentName("com.miui.securitycenter", "com.miui.permcenter.autostart.AutoStartManagementActivity"))) ||
                isIntentResolved(Intent("miui.intent.action.POWER_HIDE_MODE_APP_LIST").addCategory(Intent.CATEGORY_DEFAULT)) ||
                isIntentResolved(Intent().setComponent(ComponentName("com.miui.securitycenter", "com.miui.powercenter.PowerSettings")))
                )
    }

    //----- checkOldPreferences -------
    @PluginMethod
    fun checkOldPreferences(call: PluginCall) {
        val sPrefs: SharedPreferences = this.context.getSharedPreferences("ru.elehant.homecounters_preferences", Context.MODE_PRIVATE);
        val account_name = sPrefs.getString("account_name", null)
        val privacy_police = sPrefs.getBoolean("privacy_police", false)
        val ret = JSObject()
        ret.put("account_name", account_name)
        ret.put("privacy_police", privacy_police)
        call.resolve(ret)
    }

    //------- isMIUI -----
    @PluginMethod
    fun isMIUI(call: PluginCall)
    {
        val ret = JSObject()
            ret.put("isMIUI", checkMIUI())
        call.resolve(ret)
    }

    //------- checkBatteryOptimizationDenied -----
    @PluginMethod
    fun checkBatteryOptimizationDenied(call: PluginCall) {
        var isDenied = true;
        if (Build.VERSION.SDK_INT >= 23) {
            isDenied = powerManager?.isIgnoringBatteryOptimizations(context.packageName) ?: true
        }
        val ret = JSObject()
            ret.put("isDenied", isDenied)
        call.resolve(ret)
    }

    //----- batteryMiUIAutoStart -------
        @PluginMethod
        fun requestBatteryMiUIAutoStart(call: PluginCall) {
            val intent = Intent()
            intent.component = ComponentName("com.miui.securitycenter", "com.miui.permcenter.autostart.AutoStartManagementActivity")
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            startActivityForResult(call, intent, "requestBatteryMiUIAutoStartCallback")
        }
        @ActivityCallback
        private fun requestBatteryMiUIAutoStartCallback(call: PluginCall?, result: ActivityResult) {
            if (call == null) {
                return
            }
            call.resolve(null)
        }

    //----- requestIgnoreBatteryOptimizations -------
        @PluginMethod
        fun requestBatteryOptimisationMiUI(call: PluginCall) {
            val intent = Intent()
            .apply {
                     component = ComponentName("com.miui.powerkeeper", "com.miui.powerkeeper.ui.HiddenAppsConfigActivity")
                     putExtra("package_name", getContext().packageName)
                   // putExtra("package_label", "Счетчики Элехант")//getContext().resources.getString(R.string.app_name))
                 }
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            startActivityForResult(call, intent, "requestBatteryOptimisationMiUICallback")
        }
        @ActivityCallback
        private fun requestBatteryOptimisationMiUICallback(call: PluginCall?, result: ActivityResult) {
            if (call == null) {
                return
            }
            call.resolve(null)
        }

    //----- requestIgnoreBatteryOptimizations -------
        @SuppressLint("BatteryLife")
        @RequiresApi(Build.VERSION_CODES.M)
        @PluginMethod
        fun requestBatteryOptimizationsAll(call: PluginCall) {
            val intent = Intent(Settings.ACTION_REQUEST_IGNORE_BATTERY_OPTIMIZATIONS, Uri.parse("package:" + context.packageName))
            startActivityForResult(call, intent, "requestBatteryOptimizationsAllCallback")

        }
        @ActivityCallback
        private fun requestBatteryOptimizationsAllCallback(call: PluginCall?, result: ActivityResult) {
            if (call == null) {
                return
            }
            call.resolve(null)
        }

    //------- shouldShowRequestPermissionRationale -----
    @PluginMethod
    fun shouldShowRequestPermissionRationale(call: PluginCall)
    {
        val permission = call.getString("permission")
        val ret = JSObject()
        permission?.let {
            ret.put("shouldShowRequest",
                ActivityCompat.shouldShowRequestPermissionRationale(activity, it)
            )
        }
        call.resolve(ret)
    }

    //------- getSDKVersion -----
    @PluginMethod
    fun getSDKVersion(call: PluginCall)
    {
        val ret = JSObject()
            ret.put("SDK_INT", android.os.Build.VERSION.SDK_INT)
        call.resolve(ret)
    }

    //--------  getEmail ---
    @PluginMethod
    fun getEmail(call: PluginCall) {
        val intent = AccountPicker.newChooseAccountIntent(
                AccountPicker.AccountChooserOptions.Builder()
                        .setAllowableAccountsTypes(listOf("com.google"))
                        .build());
        startActivityForResult(call, intent, "getAccountEmailCallback")
    }

    @ActivityCallback
    private fun getAccountEmailCallback(call: PluginCall?, result: ActivityResult) {
        if (call == null) {
            return
        }
        if(result.resultCode == Activity.RESULT_OK) {
            // Do something with the result data
            result.data?.let { intent ->
                intent.getStringExtra(AccountManager.KEY_ACCOUNT_NAME)?.let { gmail ->
                    val ret = JSObject()
                    ret.put("email", gmail)
                    call.resolve(ret)
                }
            }
        } else {
            call.reject("Email canceled")
        }

    }

}
